package com.eden.room.controller;

import com.eden.room.common.Result;
import com.eden.room.common.enums.BusinessStatusEnum;
import com.eden.room.common.enums.DecorationStatusEnum;
import com.eden.room.common.enums.PropertyNatureEnum;
import com.eden.room.common.enums.RentalStatusEnum;
import com.eden.room.common.enums.RoomStatusEnum;
import com.eden.room.common.enums.RoomTypeEnum;
import com.eden.room.domain.Room;
import com.eden.room.domain.dto.RoomDTO;
import com.eden.room.domain.dto.RoomQueryDTO;
import com.eden.room.service.IRoomService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Null;
import java.util.List;
import java.util.Map;

/**
 * 房源信息控制器
 */
@RestController
@RequestMapping("/room")
@Validated
public class RoomController {

    @Autowired
    private IRoomService roomService;

    /**
     * 查询房源列表
     */
    @GetMapping("/list")
    public Result<PageInfo<Room>> list(RoomQueryDTO queryDTO,
                                      @RequestParam(defaultValue = "1") Integer pageNum,
                                      @RequestParam(defaultValue = "10") Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<Room> list = roomService.selectRoomList(queryDTO);
        return Result.success(new PageInfo<>(list));
    }

    /**
     * 获取房源详情
     */
    @GetMapping("/{id}")
    public Result<RoomDTO> getInfo(@PathVariable("id") @NotNull(message = "房源ID不能为空") Long id) {
        RoomDTO roomDTO = roomService.selectRoomDetailById(id);
        return Result.success(roomDTO);
    }

    /**
     * 新增房源
     */
    @PostMapping("/add")
    public Result<Integer> add(@RequestBody @Valid RoomDTO roomDTO) {
        return Result.success(roomService.insertRoom(roomDTO));
    }

    /**
     * 修改房源
     */
    @PostMapping("/edit")
    public Result<Integer> edit(@RequestBody @Valid RoomDTO roomDTO) {
        if (roomDTO.getId() == null) {
            return Result.error("房源ID不能为空");
        }

        // 校验枚举值，同新增

        return Result.success(roomService.updateRoom(roomDTO));
    }

    /**
     * 删除房源
     */
    @PostMapping("/remove/{id}")
    public Result<Integer> remove(@PathVariable("id") @NotNull(message = "房源ID不能为空") Long id) {
        try {
            return Result.success(roomService.deleteRoomById(id));
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
    }

    /**
     * 修改房源租赁状态
     */
    @PostMapping("/status/{id}/{rentalStatus}")
    public Result<Integer> updateRentalStatus(
            @PathVariable("id") @NotBlank(message = "房源ID不能为空") Long id,
            @PathVariable("rentalStatus") @NotBlank(message = "租赁状态不能为空") String rentalStatus) {
        
        if (!RentalStatusEnum.isValid(rentalStatus)) {
            return Result.error("无效的租赁状态");
        }
        
        try {
            return Result.success(roomService.updateRoomRentalStatus(id, rentalStatus));
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
    }
    
    /**
     * 外部接口：修改房源租赁状态
     */
    @PostMapping("/api/status")
    public Result<String> updateRentalStatusApi(
            @RequestParam("id") @NotBlank(message = "房源ID不能为空") Long id,
            @RequestParam("rentalStatus") @NotBlank(message = "租赁状态不能为空") String rentalStatus) {
        
        if (!RentalStatusEnum.isValid(rentalStatus)) {
            return Result.error("无效的租赁状态", "1");
        }
        
        try {
            roomService.updateRoomRentalStatus(id, rentalStatus);
            return Result.success();
        } catch (Exception e) {
            return Result.error(e.getMessage(), "1");
        }
    }
} 