package com.eden.room.controller;

import com.alibaba.excel.EasyExcel;
import com.eden.room.constant.RoomConstants;
import com.eden.room.domain.dto.RoomExportDTO;
import com.eden.room.domain.dto.RoomImportDTO;
import com.eden.room.domain.dto.RoomQueryDTO;
import com.eden.room.service.IBuildingService;
import com.eden.room.service.IFloorService;
import com.eden.room.service.IProjectService;
import com.eden.room.service.IRoomService;
import com.eden.room.utils.ExcelUtil;
import com.eden.room.utils.excel.BigDecimalConverter;
import com.eden.room.utils.excel.ExcelImportListener;
import com.eden.room.utils.excel.RoomImportListener;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.http.HttpHeaders;
import org.springframework.validation.Validator;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * 房源模板下载控制器
 */
@Api(tags = "房源导入导出管理", description = "提供房源模板下载、导入导出相关接口")
@RestController
@RequestMapping("/room/template")
public class RoomTemplateController {

    private static final Logger log = LoggerFactory.getLogger(RoomTemplateController.class);

    @Autowired
    private IRoomService roomService;

    @Autowired
    private IProjectService projectService;

    @Autowired
    private IBuildingService buildingService;

    @Autowired
    private IFloorService floorService;

    @Autowired
    private Validator validator;

    /**
     * 下载房源导入模板
     *
     * @param response HTTP响应对象
     */
    @ApiOperation(value = "下载旧版房源导入模板", notes = "下载旧版Excel导入模板")
    @GetMapping("/download")
    public void downloadTemplate(HttpServletResponse response) throws IOException {
        try {
            // 设置要下载的文件的名称
            String fileName = RoomConstants.ROOM_IMPORT_TEMPLATE;

            // 设置响应头
            response.setContentType(RoomConstants.CONTENT_TYPE);
            response.setCharacterEncoding(RoomConstants.CHARSET);
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.toString());
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION,
                    "attachment; filename=" + encodedFileName + ";filename*=utf-8''" + encodedFileName);

            // 获取模板文件资源
            ClassPathResource resource = new ClassPathResource("templates/excel/" + fileName);

            // 将文件内容写入响应流
            try (InputStream inputStream = resource.getInputStream();
                 OutputStream outputStream = response.getOutputStream()) {
                byte[] buffer = new byte[4096];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
                outputStream.flush();
            }
        } catch (IOException e) {
            // 如果发生异常，写入错误响应
            response.reset();
            response.setContentType("application/json");
            response.setCharacterEncoding(RoomConstants.CHARSET);
            response.getWriter().write("{\"code\":\"1\",\"message\":\"下载模板文件失败: " + e.getMessage() + "\"}");
        }
    }


    /**
     * 批量导出房源信息
     *
     * @param queryDTO 房源ID数组
     * @param response HTTP响应对象
     */
    @ApiOperation(value = "批量导出房源", notes = "根据查询条件批量导出房源详细信息为Excel文件")
    @ApiImplicitParam(name = "queryDTO", value = "queryDTO", required = true, dataType = "List")
    @PostMapping("/export/batch")
    public void exportRoomBatch(@RequestBody RoomQueryDTO queryDTO, HttpServletResponse response) {
        try {
            // 根据ID列表查询房源列表
            List<RoomExportDTO> list = roomService.exportRoom(queryDTO);

            if (list.isEmpty()) {
                throw new IllegalArgumentException("未找到指定的房源信息");
            }

            // 设置要下载的文件名称
            String fileName = "房源信息_" + System.currentTimeMillis() + ".xlsx";

            // 设置响应头
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.toString());
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION,
                    "attachment; filename=" + encodedFileName + ";filename*=utf-8''" + encodedFileName);

            // 导出Excel
            ExcelUtil<RoomExportDTO> excelUtil = new ExcelUtil<>(RoomExportDTO.class);
            excelUtil.exportExcel(list, "房源信息", response.getOutputStream());

        } catch (Exception e) {
            log.error("批量导出房源信息失败", e);
            // 如果发生异常，写入错误响应
            try {
                response.reset();
                response.setContentType("application/json");
                response.setCharacterEncoding("utf-8");
                response.getWriter().write("{\"code\":\"1\",\"message\":\"批量导出房源信息失败: " + e.getMessage() + "\"}");
            } catch (IOException ioException) {
                log.error("写入错误响应失败", ioException);
            }
        }
    }

    /**
     * 批量导入房源数据
     *
     * @param file 导入房源的Excel文件（必须遵循指定的模板格式）
     * @param response HTTP响应对象
     */
    @ApiOperation(value = "批量导入房源", notes = "通过Excel文件批量导入房源数据，返回导入结果信息")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true, dataType = "MultipartFile")
    @PostMapping("/import")
    public void importRoom(@RequestParam("file") MultipartFile file, HttpServletResponse response) throws IOException {
        // 校验文件
        validateFile(file);

        log.info("开始导入Excel文件: {}, 大小: {} bytes", file.getOriginalFilename(), file.getSize());

        // 创建导入监听器
        RoomImportListener listener = new RoomImportListener(validator, roomService, projectService, buildingService, floorService);

        // 读取Excel文件，添加自定义转换器
        EasyExcel.read(file.getInputStream(), RoomImportDTO.class, listener)
            .registerConverter(new BigDecimalConverter())
            .headRowNumber(1)
            .sheet()
            .doRead();

        // 获取错误信息
        Boolean successFlag = listener.getSuccessFlag();

        if (successFlag) {
            // 如果没有错误，直接返回成功
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            response.getWriter().write("{\"code\":\"0\",\"message\":\"导入成功\"}");
            return;
        }

        // 如果有错误，生成带错误信息的Excel文件
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        String encodedFileName = URLEncoder.encode("room_import_result.xlsx", StandardCharsets.UTF_8.toString());
        response.setHeader("Content-Disposition", "attachment;filename=" + encodedFileName);

        // 写入带错误信息的Excel文件
        EasyExcel.write(response.getOutputStream(), RoomImportDTO.class)
                .sheet("导入结果")
                .doWrite(listener.getDataList());
    }


    /**
     * 校验文件
     */
    private void validateFile(MultipartFile file) {
        if (file == null || file.isEmpty()) {
            throw new RuntimeException("请选择要上传的文件");
        }

        // 校验文件大小
        if (file.getSize() > RoomConstants.MAX_FILE_SIZE) {
            throw new RuntimeException("文件大小不能超过1MB");
        }

        // 校验文件格式
        String filename = file.getOriginalFilename();
        if (filename == null || !(filename.toLowerCase().endsWith(".xls") || filename.toLowerCase().endsWith(".xlsx"))) {
            throw new RuntimeException("文件格式不正确，请上传Excel文件");
        }
    }
} 