package com.eden.room.service.impl;


import com.eden.room.common.ContextUser;
import com.eden.room.common.UserContextHolder;
import com.eden.room.common.enums.*;
import com.eden.room.domain.*;
import com.eden.room.domain.dto.*;
import com.eden.room.feign.ContractFeignClient;
import com.eden.room.feign.CustomerFeignClient;
import com.eden.room.feign.TagFeignClient;
import com.eden.room.mapper.BuildingMapper;
import com.eden.room.mapper.FloorMapper;
import com.eden.room.mapper.ProjectMapper;
import com.eden.room.mapper.RoomBusinessInfoMapper;
import com.eden.room.mapper.RoomExtendInfoMapper;
import com.eden.room.mapper.RoomImageMapper;
import com.eden.room.mapper.RoomMapper;
import com.eden.room.service.IRoomService;
import com.eden.room.utils.EnumConverter;
import com.eden.room.utils.ExcelUtil;
import com.eden.room.utils.excel.Excel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;

import com.eden.room.constant.RoomConstants;

/**
 * 房源服务实现类
 */
@Service
@Slf4j
public class RoomServiceImpl implements IRoomService {

    @Autowired
    private RoomMapper roomMapper;
    
    @Autowired
    private RoomBusinessInfoMapper businessInfoMapper;
    
    @Autowired
    private RoomExtendInfoMapper extendInfoMapper;
    
    @Autowired
    private RoomImageMapper imageMapper;
    
    @Autowired
    private ProjectMapper projectMapper;
    
    @Autowired
    private BuildingMapper buildingMapper;
    
    @Autowired
    private FloorMapper floorMapper;
    
    @Autowired
    private TagFeignClient tagFeignClient;
    
    @Autowired
    private CustomerFeignClient customerFeignClient;
    
    @Autowired
    private ContractFeignClient contractFeignClient;

    /**
     * 查询房源列表
     */
    @Override
    public List<Room> selectRoomList(RoomQueryDTO queryDTO) {
        return roomMapper.selectRoomList(queryDTO);
    }

    /**
     * 根据ID查询房源
     */
    @Override
    public Room selectRoomById(Long id) {
        return roomMapper.selectRoomById(id);
    }

    /**
     * 查询房源详情（包括基础信息、招商信息、扩展信息）
     */
    @Override
    public RoomDTO selectRoomDetailById(Long id) {
        // 查询房源基础信息
        Room room = roomMapper.selectRoomById(id);
        if (room == null) {
            return null;
        }
        
        RoomDTO roomDTO = new RoomDTO();
        BeanUtils.copyProperties(room, roomDTO);
        
        // 查询招商信息
        RoomBusinessInfo businessInfo = businessInfoMapper.selectRoomBusinessInfoByRoomId(id);
        roomDTO.setBusinessInfo(businessInfo);
        
        // 查询拓展信息
        RoomExtendInfo extendInfo = extendInfoMapper.selectRoomExtendInfoByRoomId(id);
        roomDTO.setExtendInfo(extendInfo);
        
        // 查询房源图片
        List<RoomImage> images = imageMapper.selectRoomImagesByRoomId(id);
        if (images != null && !images.isEmpty()) {
            // 过滤房源图片
            List<RoomImage> roomImages = images.stream()
                    .filter(img -> ImageTypeEnum.ROOM_IMAGE.getValue().equals(img.getImageType()))
                    .collect(Collectors.toList());
            roomDTO.setRoomImages(roomImages);
            
            // 过滤户型图
            List<RoomImage> floorPlanImages = images.stream()
                    .filter(img -> ImageTypeEnum.FLOOR_PLAN.getValue().equals(img.getImageType()))
                    .collect(Collectors.toList());
            roomDTO.setFloorPlanImages(floorPlanImages);
        }
        
        // 查询项目名称
        if (StringUtils.isNotEmpty(room.getProjectId())) {
            roomDTO.setProjectName(projectMapper.selectProjectById(room.getProjectId()).getProjectName());
        }
        
        // 查询楼宇名称
        if (room.getBuildingId()!=null) {
            roomDTO.setBuildingName(buildingMapper.selectBuildingById(room.getBuildingId()).getBuildingName());
        }
        
        // 查询楼层名称
        if (room.getFloorId()!=null) {
            roomDTO.setFloorName(floorMapper.selectFloorById(room.getFloorId()).getFloorName());
        }
        
        // 查询业主名称
        if (room.getOwnerId()!=null) {
            OwnerDTO owner = customerFeignClient.getOwnerById(room.getOwnerId());
            if (owner != null) {
                roomDTO.setOwnerName(owner.getName());
            }
        }
        
        // 查询标签
        if (extendInfo != null && StringUtils.isNotEmpty(extendInfo.getTags())) {
            String[] tagIds = extendInfo.getTags().split(",");
            List<TagDTO> tags = tagFeignClient.getTagsByIds(tagIds);
            roomDTO.setTags(roomDTO.getTags());
        }
        
        return roomDTO;
    }

    /**
     * 新增房源信息
     */
    @Override
    @Transactional
    public int insertRoom(RoomDTO roomDTO) {
        // 获取当前用户
        ContextUser user = new ContextUser();
        user.setUsername(RoomConstants.DEFAULT_USERNAME);

        Date now = new Date();
        
        // 设置房源基础信息
        Room room = new Room();
        BeanUtils.copyProperties(roomDTO, room);
        room.setCreateTime(now);
        room.setLastModTime(now);
        room.setCreateUserId(user.getUserName());
        room.setLastModUserId(user.getUserName());
        room.setDelFlag(RoomConstants.DEL_FLAG_NORMAL);

        // todo 数据库校验
        roomMapper.checkRoomDigitalNumberExist(room.getRoomDigitalNumber(), room.getProjectId(),
                room.getBuildingId(), room.getFloorId());

        // todo 数据库校验
        roomMapper.checkRoomNumberExist(room.getRoomNumber(), room.getProjectId(),
                room.getBuildingId(), room.getFloorId());

        // 插入房源基础信息
        int rows = roomMapper.batchInsertRooms(Collections.singletonList(room));
        Long roomId = room.getId();
        
        // 设置招商信息
        RoomBusinessInfo businessInfo = roomDTO.getBusinessInfo();
        if (businessInfo != null) {
            businessInfo.setRoomId(roomId);
            businessInfo.setCreateTime(now);
            businessInfo.setLastModTime(now);
            businessInfo.setCreateUserId(user.getUserName());
            businessInfo.setLastModUserId(user.getUserName());
            businessInfo.setDelFlag(RoomConstants.DEL_FLAG_NORMAL);
            
            // 默认设置为待租状态
            if (StringUtils.isEmpty(businessInfo.getRentalStatus())) {
                businessInfo.setRentalStatus(RentalStatusEnum.WAITING.getKey());
            }
            businessInfo.setPriceUnit(businessInfo.getPriceUnit());
            // 插入招商信息
            businessInfoMapper.batchInsertBusinessInfos(Collections.singletonList(businessInfo));
        }
        
        // 设置拓展信息
        RoomExtendInfo extendInfo = roomDTO.getExtendInfo();
        if (extendInfo != null) {
            extendInfo.setRoomId(roomId);
            extendInfo.setCreateTime(now);
            extendInfo.setLastModTime(now);
            extendInfo.setCreateUserId(user.getUserName());
            extendInfo.setLastModUserId(user.getUserName());
            extendInfo.setDelFlag(RoomConstants.DEL_FLAG_NORMAL);
            
            // 处理标签
            if (roomDTO.getTags() != null && !roomDTO.getTags().isEmpty()) {
                extendInfo.setTags(roomDTO.getTags());
            }
            
            // 插入拓展信息
            extendInfoMapper.batchInsertExtendInfos(Collections.singletonList(extendInfo));
        }
        
        // 插入房源图片
        insertRoomImages(roomDTO.getRoomImages(), roomId, ImageTypeEnum.ROOM_IMAGE.getValue(), user);
        
        // 插入户型图
        insertRoomImages(roomDTO.getFloorPlanImages(), roomId, ImageTypeEnum.FLOOR_PLAN.getValue(), user);
        
        return rows;
    }

    /**
     * 修改房源信息
     */
    @Override
    @Transactional
    public int updateRoom(RoomDTO roomDTO) {
        // 获取当前用户
        ContextUser user = new ContextUser();
        user.setUsername(RoomConstants.DEFAULT_USERNAME);
        Date now = new Date();
        Long roomId = roomDTO.getId();
        
        // 设置房源基础信息
        Room room = new Room();
        BeanUtils.copyProperties(roomDTO, room);
        room.setLastModTime(now);
        room.setLastModUserId(user.getUserName());
        
        // 更新房源基础信息
        int rows = roomMapper.updateRoom(room);
        
        // 更新招商信息
        RoomBusinessInfo businessInfo = roomDTO.getBusinessInfo();
        if (businessInfo != null) {
            businessInfo.setRoomId(roomId);
            businessInfo.setLastModTime(now);
            businessInfo.setLastModUserId(user.getUserName());
            
            // 判断招商信息是否存在
            RoomBusinessInfo existBusinessInfo = businessInfoMapper.selectRoomBusinessInfoByRoomId(roomId);
            if (existBusinessInfo != null) {
                businessInfoMapper.updateRoomBusinessInfo(businessInfo);
            } else {
                businessInfo.setCreateTime(now);
                businessInfo.setCreateUserId(user.getUserName());
                businessInfo.setDelFlag(RoomConstants.DEL_FLAG_NORMAL);
                businessInfoMapper.batchInsertBusinessInfos(Collections.singletonList(businessInfo));
            }
        }
        
        // 更新拓展信息
        RoomExtendInfo extendInfo = roomDTO.getExtendInfo();
        if (extendInfo != null) {
            extendInfo.setRoomId(roomId);
            extendInfo.setLastModTime(now);
            extendInfo.setLastModUserId(user.getUserName());
            
            // 处理标签
            if (roomDTO.getTags() != null) {
                extendInfo.setTags(roomDTO.getTags());
            }
            
            // 判断拓展信息是否存在
            RoomExtendInfo existExtendInfo = extendInfoMapper.selectRoomExtendInfoByRoomId(roomId);
            if (existExtendInfo != null) {
                extendInfoMapper.updateRoomExtendInfo(extendInfo);
            } else {
                extendInfo.setCreateTime(now);
                extendInfo.setCreateUserId(user.getUserName());
                extendInfo.setDelFlag(RoomConstants.DEL_FLAG_NORMAL);
                extendInfoMapper.batchInsertExtendInfos(Collections.singletonList(extendInfo));
            }
        }
        
        // 更新房源图片：先删除再新增
        if (roomDTO.getRoomImages() != null || roomDTO.getFloorPlanImages() != null) {
            // 删除原有图片
            imageMapper.deleteRoomImagesByRoomId(roomId);
            
            // 重新插入房源图片
            insertRoomImages(roomDTO.getRoomImages(), roomId, ImageTypeEnum.ROOM_IMAGE.getValue(), user);
            
            // 重新插入户型图
            insertRoomImages(roomDTO.getFloorPlanImages(), roomId, ImageTypeEnum.FLOOR_PLAN.getValue(), user);
        }
        
        return rows;
    }

    /**
     * 删除房源信息
     */
    @Override
    @Transactional
    public int deleteRoomById(Long id) {
        // 检查房源是否有正在执行的合同
        if (checkRoomContract(id)) {
            throw new RuntimeException("该房源有正在执行的合同，不能删除");
        }
        
        // 删除房源基础信息
        int rows = roomMapper.deleteRoomById(id);
        
        // 删除招商信息
        businessInfoMapper.deleteRoomBusinessInfoByRoomId(id);

        // 删除拓展信息
        extendInfoMapper.deleteRoomExtendInfoByRoomId(id);
        
        // 删除图片信息
        imageMapper.deleteRoomImagesByRoomId(id);
        
        return rows;
    }


    /**
     * 导出房源信息
     */
    @Override
    public List<RoomExportDTO> exportRoom(RoomQueryDTO queryDTO) {

        // 查询房源信息
        List<Room> roomList = roomMapper.selectRoomList(queryDTO);
        if (roomList == null || roomList.isEmpty()) {
            return new ArrayList<>();
        }
        
        // 转换为导出DTO
        List<RoomExportDTO> exportList = new ArrayList<>();
        if (roomList.isEmpty()) {
            return exportList;
        }
        // 转换为导出DTO
        for (Room r : roomList) {
            RoomExportDTO exportDTO = new RoomExportDTO();

            // 复制基本信息
            exportDTO.setId(r.getId());
            exportDTO.setProjectId(r.getProjectId());
            exportDTO.setProjectName(r.getProjectName());
            exportDTO.setBuildingId(r.getBuildingId());
            exportDTO.setBuildingName(r.getBuildingName());
            exportDTO.setFloorId(r.getFloorId());
            exportDTO.setFloorName(r.getFloorName());
            exportDTO.setRoomNumber(r.getRoomNumber());
            exportDTO.setRoomDigitalNumber(r.getRoomDigitalNumber());
            exportDTO.setRoomManageNumber(r.getRoomManageNumber());
            exportDTO.setRoomName(r.getRoomName());
            exportDTO.setRoomType(r.getRoomType());
            exportDTO.setFloor(r.getFloor());
            exportDTO.setBuildingArea(r.getBuildingArea());
            exportDTO.setRentalArea(r.getRentalArea());
            exportDTO.setBillingArea(r.getBillingArea());
            exportDTO.setInnerArea(r.getInnerArea());
            exportDTO.setIsVirtual(r.getIsVirtual());
            exportDTO.setDeliveryTime(r.getDeliveryTime());
            exportDTO.setOwnerId(r.getOwnerId());
            exportDTO.setPropertyNature(r.getPropertyNature());
            exportDTO.setHeight(r.getHeight());
            exportDTO.setLoadValue(r.getLoadValue());
            exportDTO.setRoomStatus(r.getRoomStatus());
            exportDTO.setCreateTime(r.getCreateTime());
            exportDTO.setPrice(r.getPrice());
            exportDTO.setPriceUnit(r.getPriceUnit());
            exportDTO.setRentalStatus(r.getRentalStatus());
            exportDTO.setAvailableDate(r.getAvailableDate());
            exportDTO.setBusinessStatus(r.getBusinessStatus());
            exportDTO.setDecorationStatus(r.getDecorationStatus());
            // todo 查询接口进行转换 注意不要在for循环里面调用
            exportDTO.setTags(r.getTags());
            exportDTO.setPriceAndUnit((exportDTO.getPrice()));

            // 转换枚举字段
            EnumConverter.convert(exportDTO);

            exportList.add(exportDTO);
        }
        
        return exportList;
    }

    /**
     * 修改房源租赁状态
     */
    @Override
    public int updateRoomRentalStatus(Long id, String rentalStatus) {
        // 校验租赁状态是否有效
        if (!RentalStatusEnum.isValid(rentalStatus)) {
            throw new RuntimeException("无效的租赁状态");
        }
        
        // 查询招商信息
        RoomBusinessInfo businessInfo = businessInfoMapper.selectRoomBusinessInfoByRoomId(id);
        if (businessInfo == null) {
            throw new RuntimeException("房源招商信息不存在");
        }
        
        // 设置租赁状态
        businessInfo.setRentalStatus(rentalStatus);
        businessInfo.setLastModTime(new Date());
        businessInfo.setLastModUserId(UserContextHolder.get().getUserName());
        
        return businessInfoMapper.updateRoomBusinessInfo(businessInfo);
    }

    /**
     * 检查房源是否有正在执行的合同
     */
    @Override
    public boolean checkRoomContract(Long id) {
        // 调用合同服务接口，检查房源是否有正在执行的合同
        return contractFeignClient.checkRoomContract(id, null);
    }

    /**
     * 插入房源图片
     *
     * @param images 图片列表
     * @param roomId 房源ID
     * @param imageType 图片类型
     * @param user 当前用户
     */
    private void insertRoomImages(List<RoomImage> images, Long roomId, String imageType, ContextUser user) {
        if (images != null && !images.isEmpty()) {
            Date now = new Date();
            int sortOrder = 0;
            
            for (RoomImage image : images) {
                image.setRoomId(roomId);
                image.setImageType(imageType);
                image.setSortOrder(sortOrder++);
                image.setCreateTime(now);
                image.setLastModTime(now);
                image.setCreateUserId(user.getUserName());
                image.setLastModUserId(user.getUserName());
                image.setDelFlag(RoomConstants.DEL_FLAG_NORMAL);
                
                imageMapper.insertRoomImage(image);
            }
        }
    }

    private List<Field> getExcelFields(Class<?> clazz) {
        List<Field> fields = new ArrayList<>();
        Field[] declaredFields = clazz.getDeclaredFields();
        for (Field field : declaredFields) {
            if (field.isAnnotationPresent(Excel.class)) {
                fields.add(field);
            }
        }
        // 按照sort属性排序
        fields.sort((f1, f2) -> {
            Excel excel1 = f1.getAnnotation(Excel.class);
            Excel excel2 = f2.getAnnotation(Excel.class);
            return excel1.sort() - excel2.sort();
        });
        return fields;
    }
    /**
     * 根据项目名称获取项目ID
     *
     * @param projectName 项目名称
     * @return 项目ID
     */
    private String getProjectIdByName(String projectName) {
        if (StringUtils.isBlank(projectName)) {
            return null;
        }
        Project param = new Project();
        param.setProjectName(projectName);
        return projectMapper.selectProjectIdByName(projectName);
    }
    
    /**
     * 根据楼宇名称获取楼宇ID
     *
     * @param projectId 项目ID
     * @param buildingName 楼宇名称
     * @return 楼宇ID
     */
    private Long getBuildingIdByName(String projectId, String buildingName) {
        if (StringUtils.isBlank(projectId) || StringUtils.isBlank(buildingName)) {
            return null;
        }
        
        Building param = new Building();
        param.setProjectId(projectId);
        param.setBuildingName(buildingName);
        List<Building> buildings = buildingMapper.selectBuildingList(param);
        return buildings.isEmpty() ? null : buildings.get(0).getId();
    }
    
    /**
     * 根据楼层名称获取楼层ID
     *
     * @param buildingId 楼宇ID
     * @param floorName 楼层名称
     * @return 楼层ID
     */
    private Long getFloorIdByName(Long buildingId, String floorName) {
        if (buildingId == null || StringUtils.isBlank(floorName)) {
            return null;
        }
        
        Floor param = new Floor();
        param.setBuildingId(buildingId);
        param.setFloorName(floorName);
        List<Floor> floors = floorMapper.selectFloorList(param);
        return floors.isEmpty() ? null : floors.get(0).getId();
    }
    
    /**
     * 检查房源编号是否已存在
     *
     * @param roomCode 房源编号
     * @param projectId 项目ID
     * @param buildingId 楼宇ID
     * @param floorId 楼层ID
     * @return 是否存在
     */
    private boolean checkRoomCodeExists(String roomCode, String projectId, Long buildingId, Long floorId) {
        if (StringUtils.isBlank(roomCode)) {
            return false;
        }

        RoomQueryDTO param = new RoomQueryDTO();
        param.setRoomNumber(roomCode);
        param.setProjectId(projectId);
        param.setBuildingId(buildingId);
        param.setFloorId(floorId);
        List<Room> rooms = roomMapper.selectRoomList(param);
        return !rooms.isEmpty();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchInsertRooms(List<Room> rooms, List<RoomBusinessInfo> businessInfos, List<RoomExtendInfo> extendInfos) {
        if (rooms == null || rooms.isEmpty()) {
            return;
        }
        
        // 设置批量插入的批次大小
        int batchSize = RoomConstants.BATCH_SIZE;
    
        // 分批处理房间信息
        for (int i = 0; i < rooms.size(); i += batchSize) {
            int end = Math.min(i + batchSize, rooms.size());
            List<Room> batch = rooms.subList(i, end);
            roomMapper.batchInsertRooms(batch);
        }
        
        // 获取所有插入的Room的ID
        List<Long> roomIds = rooms.stream()
                .map(Room::getId)
                .collect(Collectors.toList());
        
        // 设置RoomBusinessInfo和RoomExtendInfo的roomId
        if (businessInfos != null && !businessInfos.isEmpty()) {
            for (int i = 0; i < businessInfos.size(); i++) {
                if (i < roomIds.size()) {
                    businessInfos.get(i).setRoomId(roomIds.get(i));
                }
            }
            
            // 分批处理招商信息
            for (int i = 0; i < businessInfos.size(); i += batchSize) {
                int end = Math.min(i + batchSize, businessInfos.size());
                List<RoomBusinessInfo> batch = businessInfos.subList(i, end);
                businessInfoMapper.batchInsertBusinessInfos(batch);
            }
        }

        // 设置RoomExtendInfo的roomId并分批处理扩展信息
        if (extendInfos != null && !extendInfos.isEmpty()) {
            for (int i = 0; i < extendInfos.size(); i++) {
                if (i < roomIds.size()) {
                    extendInfos.get(i).setRoomId(roomIds.get(i));
                }
            }
            
            for (int i = 0; i < extendInfos.size(); i += batchSize) {
                int end = Math.min(i + batchSize, extendInfos.size());
                List<RoomExtendInfo> batch = extendInfos.subList(i, end);
                extendInfoMapper.batchInsertExtendInfos(batch);
            }
        }
    }
    @Override
    public Set<String> getExistingRoomNumbers(String projectId, Set<String> roomNumbers) {
        if (projectId == null || roomNumbers == null || roomNumbers.isEmpty()) {
            return Collections.emptySet();
        }

        // 1. 查询数据库中已存在的房间号
        Set<String> existingInDB = roomMapper.getExistingRoomNumbers(projectId, roomNumbers);

        // 2. 检查当前批次中是否有重复的房间号
        Set<String> duplicatesInBatch = new HashSet<>();
        Set<String> seen = new HashSet<>();

        for (String roomNumber : roomNumbers) {
            if (!seen.add(roomNumber)) {
                duplicatesInBatch.add(roomNumber);
            }
        }

        // 3. 合并所有重复的房间号
        Set<String> allDuplicates = new HashSet<>(existingInDB);
        allDuplicates.addAll(duplicatesInBatch);

        return allDuplicates;
    }

}