package com.eden.room.utils.excel;

import com.alibaba.excel.context.AnalysisContext;
import com.eden.room.common.enums.*;
import com.eden.room.domain.Room;
import com.eden.room.domain.RoomBusinessInfo;
import com.eden.room.domain.RoomExtendInfo;
import com.eden.room.domain.dto.RoomImportDTO;
import com.eden.room.service.IBuildingService;
import com.eden.room.service.IFloorService;
import com.eden.room.service.IProjectService;
import com.eden.room.service.IRoomService;
import org.springframework.validation.BeanPropertyBindingResult;
import org.springframework.validation.BindingResult;
import org.springframework.validation.Validator;
import com.eden.room.constant.RoomConstants;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 房源导入监听器
 */
public class RoomImportListener extends ExcelImportListener<RoomImportDTO> {


    private final IRoomService roomService;

    private final IProjectService projectService;

    private final IBuildingService buildingService;

    private final IFloorService floorService;

    // 用于存储待导入的数据
    private final List<Room> rooms = new ArrayList<>();
    private final List<RoomBusinessInfo> businessInfos = new ArrayList<>();
    private final List<RoomExtendInfo> extendInfos = new ArrayList<>();

    public RoomImportListener(Validator validator, IRoomService roomService, IProjectService projectService, IBuildingService buildingService, IFloorService floorService) {
        super(validator);
        this.roomService = roomService;
        this.projectService = projectService;
        this.buildingService = buildingService;
        this.floorService = floorService;
    }

    @Override
    public void invoke(RoomImportDTO data, AnalysisContext context) {
        // 获取当前行号
        int rowIndex = context.readRowHolder().getRowIndex();
        if (rowIndex <= RoomConstants.IMPORT_HEADER_ROW_INDEX) {
            return;
        }
        dataList.add(data);
        // 数据校验
        BindingResult bindingResult = new BeanPropertyBindingResult(data, RoomImportDTO.class.getName());
        validator.validate(data, bindingResult);

        if (bindingResult.hasErrors()) {
            // 记录错误信息
            data.setImportResult(bindingResult.getAllErrors().get(0).getDefaultMessage());
            successFlag = false;
        }

    }
    @Override
    protected void batchProcess() {
        if (dataList.isEmpty()) {
            return;
        }

        Date now = new Date();

        // 1. 收集所有需要查询的项目、楼宇、楼层信息
        Set<String> projectNames = dataList.stream()
                .map(RoomImportDTO::getProjectName)
                .collect(Collectors.toSet());

        Map<String, Set<String>> projectBuildingMap = new HashMap<>();
        Map<String, Set<String>> buildingFloorMap = new HashMap<>();

        for (RoomImportDTO dto : dataList) {
            projectBuildingMap.computeIfAbsent(dto.getProjectName(), k -> new HashSet<>())
                    .add(dto.getBuildingName());
            buildingFloorMap.computeIfAbsent(dto.getBuildingName(), k -> new HashSet<>())
                    .add(dto.getFloorName());
        }

        // 2. 批量查询项目、楼宇、楼层信息
        Map<String, String> projectIdMap = projectService.getProjectIdsByNames(projectNames);
        Map<String, Map<String, Long>> buildingIdMap = new HashMap<>();
        Map<String, Map<String, Long>> floorIdMap = new HashMap<>();

        for (Map.Entry<String, Set<String>> entry : projectBuildingMap.entrySet()) {
            String projectId = projectIdMap.get(entry.getKey());
            if (projectId != null) {
                buildingIdMap.put(projectId,
                        buildingService.getBuildingIdsByNames(projectId, entry.getValue()));
            }
        }

        for (Map.Entry<String, Set<String>> entry : buildingFloorMap.entrySet()) {
            for (Map<String, Long> buildingMap : buildingIdMap.values()) {
                Long buildingId = buildingMap.get(entry.getKey());
                if (buildingId != null) {
                    floorIdMap.put(buildingId.toString(),
                            floorService.getFloorIdsByNames(buildingId, entry.getValue()));
                }
            }
        }

        // 3. 收集所有需要检查的房间号
        Map<String, Set<String>> projectRoomMap = new HashMap<>();
        for (RoomImportDTO dto : dataList) {
            String projectId = projectIdMap.get(dto.getProjectName());
            if (projectId != null) {
                projectRoomMap.computeIfAbsent(projectId, k -> new HashSet<>())
                        .add(dto.getRoomNumber());
            }
        }

        // 4. 批量查询已存在的房间号
        Map<String, Set<String>> existingRoomMap = new HashMap<>();
        for (Map.Entry<String, Set<String>> entry : projectRoomMap.entrySet()) {
            existingRoomMap.put(entry.getKey(),
                roomService.getExistingRoomNumbers(entry.getKey(), entry.getValue()));
        }

        // 5. 处理每一行数据
        for (RoomImportDTO dto : dataList) {

                String projectId = projectIdMap.get(dto.getProjectName());
                if (projectId == null) {
                    dto.setImportResult("项目不存在。");
                    successFlag = false;
                }

                Map<String, Long> buildingMap = buildingIdMap.get(projectId);
                Long buildingId = buildingMap != null ? buildingMap.get(dto.getBuildingName()) : null;
                if (buildingId == null) {
                    dto.setImportResult("楼宇不存在。");
                    successFlag = false;
                }

                Map<String, Long> floorMap = floorIdMap.get(buildingId.toString());
                Long floorId = floorMap != null ? floorMap.get(dto.getFloorName()) : null;
                if (floorId == null) {
                    dto.setImportResult("楼层不存在。");
                    successFlag = false;
                }

                // 检查房间号是否重复
                Set<String> existingRooms = existingRoomMap.get(projectId);
                if (existingRooms != null && existingRooms.contains(dto.getRoomNumber())) {
                    dto.setImportResult("房间号已存在。");
                    successFlag = false;
                }

                if (!successFlag){
                    continue;
                }

                // 创建Room对象
                Room room = new Room();
                room.setProjectId(projectId);
                room.setProjectName(dto.getProjectName());
                room.setBuildingId(buildingId);
                room.setBuildingName(dto.getBuildingName());
                room.setFloorId(floorId);
                room.setFloorName(dto.getFloorName());
                room.setRoomNumber(dto.getRoomNumber());
                room.setRoomDigitalNumber(dto.getRoomDigitalNumber());
                room.setRoomManageNumber(dto.getRoomManageNumber());
                room.setBuildingArea(dto.getBuildingArea());
                room.setRentalArea(dto.getRentalArea());
                room.setBillingArea(dto.getBillingArea());
                room.setHeight(dto.getHeight());
                room.setIsVirtual(VirtualFlagEnum.NO.getKey());
                room.setRoomStatus(RoomStatusEnum.PUBLIC.getKey());
                room.setDelFlag(RoomConstants.DEL_FLAG_NORMAL);
                room.setCreateTime(now);
                room.setLastModTime(now);
                rooms.add(room);

                // 创建RoomBusinessInfo对象
                RoomBusinessInfo businessInfo = new RoomBusinessInfo();
                businessInfo.setPrice(dto.getPrice());
                businessInfo.setFloorPrice(dto.getFloorPrice());
                businessInfo.setPriceUnit(PriceUnitEnum.getByValue(dto.getPriceUnit()));
                businessInfo.setRentalStatus(RentalStatusEnum.WAITING.getKey());
                businessInfo.setBusinessStatus(BusinessStatusEnum.BUSINESS.getKey());
                businessInfo.setDecorationStatus(DecorationStatusEnum.ROUGH.getKey());
                businessInfo.setDelFlag(RoomConstants.DEL_FLAG_NORMAL);
                businessInfo.setCreateTime(now);
                businessInfo.setLastModTime(now);
                businessInfos.add(businessInfo);

                // 创建RoomExtendInfo对象
                RoomExtendInfo extendInfo = new RoomExtendInfo();
                extendInfo.setTags(dto.getTags());
                extendInfo.setDelFlag(RoomConstants.DEL_FLAG_NORMAL);
                extendInfo.setCreateTime(now);
                extendInfo.setLastModTime(now);
                extendInfos.add(extendInfo);

        }

        // 6. 批量保存数据
        if (!rooms.isEmpty() && successFlag) {
            roomService.batchInsertRooms(rooms, businessInfos, extendInfos);
        }
    }

    @Override
    public List<RoomImportDTO> getDataList() {
        return dataList;
    }

}